@@ -52,12 +52,16 @@ Let's look at the sections of this file.
 import os
 import re
 from enum import Enum
-from dataclasses import dataclass
 from typing import List, Dict, Union
+from dataclasses import dataclass, field
 
 import yaml
 
+# pylint: disable=unused-import
+import intents # Needed to generate docs
 from intents.language import agent_language, LanguageCode
+# from intents.model.agent import _AgentMetaclass
+from intents.model.intent import Intent, _IntentMetaclass
 from intents.model.entity import _EntityMetaclass
 
 #
@@ -95,11 +99,11 @@ class ExampleUtterance(str):
     """
     
     # TODO: check for escape characters - intent is possibly intent_cls
-    def __init__(self, example: str, intent: "intents.Intent"):
+    def __init__(self, example: str, intent: Intent):
         self._intent = intent
         self.chunks() # Will check parameters
     
-    def __new__(cls, example: str, intent: "intents.Intent"):
+    def __new__(cls, example: str, intent: Intent):
         return super().__new__(cls, example)
 
     def chunks(self):
@@ -117,7 +121,7 @@ class ExampleUtterance(str):
 
         TODO: handle escaping
         """
-        parameter_schema = self._intent.parameter_schema()
+        parameter_schema = self._intent.parameter_schema
         result = []
         last_end = 0
         for m in RE_EXAMPLE_PARAMETERS.finditer(self):
@@ -170,6 +174,9 @@ class IntentResponse:
         YAML file. Typically, IntentResponse is a dataclass and `data` is a dict
         of fields; however specific subclasses may override with custom
         parameters.
+
+        Args:
+            data: A response object, as it is read from the YAML file
         """
         return cls(**data)
 
@@ -197,6 +204,10 @@ class TextIntentResponse(IntentResponse):
             - text:
               - This is a response
               - This is an alternative response
+
+    Args:
+        choices: A list of equivalent responses. One will be chosen at random at
+            prediction time
     """
 
     choices: List[str]
@@ -230,6 +241,9 @@ class QuickRepliesIntentResponse(IntentResponse):
           - quick_replies:
             - Order Pizza
             - Order Beer
+
+    Args:
+        replies: A list of possible User replies to the current intent 
     """
 
     replies: List[str]
@@ -268,6 +282,10 @@ class ImageIntentResponse(IntentResponse):
           - image:
               url: https://example.com/image.png
               title: An example image
+    
+    Args:
+        url: A publicly accessible image URL
+        title: A title for the image
     """
     url: str
     title: str = None
@@ -297,6 +315,12 @@ class CardIntentResponse(IntentResponse):
               subtitle: An optional subtitle
               image: https://example.com/image.jpeg
               link: https://example.com/
+
+    Args:
+        title: The card title
+        subtitle: A short subtitle
+        image: URL of an image to be used as cover
+        link: A link to follow if User taps the card
     """
     title: str
     subtitle: str = None
@@ -310,6 +334,10 @@ class CustomPayloadIntentResponse(IntentResponse):
     Dialogflow in every response group, including "Default". Currently they can
     only be defined in the YAML as free form payloads; support for marshalling
     or generation from code is expected in future developments.
+
+    Args:
+        name: A name that identifies the payload type
+        payload: Any JSON-serializable object
     """
 
     name: str
@@ -365,14 +393,22 @@ class IntentLanguageData:
     responses that include cards, images and quick replies.
 
     **Slot Filling Promps** are used to solve parameters that couldn't be tagged in
-    the original message. For instance a `order_pizza` intent may have a
+    the original message. For instance a `OrderPizza` intent may have a
     `pizza_type` parameter. When User asks "I'd like a pizza" we want to fill
     the slot by asking "What type of pizza?". `slot_filling_prompts` will map
     parameters to their prompts: `{"pizza_type": ["What type of pizza?"]}`
+
+    Args:
+        example_utterance: A list of User utterances that should be associated
+            with the intent
+        slot_filling_prompts: A map of prompts to use when required parameters
+            are missing
+        responses: The Responses that Agent will send to User when the intent is
+            predicted 
     """
-    example_utterances: List[ExampleUtterance]
-    slot_filling_prompts: Dict[str, List[str]]
-    responses: Dict[IntentResponseGroup, List[IntentResponse]]
+    example_utterances: List[ExampleUtterance] = field(default_factory=list)
+    slot_filling_prompts: Dict[str, List[str]] = field(default_factory=dict)
+    responses: Dict[IntentResponseGroup, List[IntentResponse]] = field(default_factory=list)
 
 #
 # Language Data Loader
@@ -380,9 +416,21 @@ class IntentLanguageData:
 
 def intent_language_data(
     agent_cls: "intents.model.agent._AgentMetaclass",
-    intent_cls: "intents.model.intent._IntentMetaclass",
+    intent_cls: _IntentMetaclass,
     language_code: LanguageCode=None
 ) -> Dict[LanguageCode, IntentLanguageData]:
+    """
+    Return Language Data for the given Intent.
+
+    Args:
+        agent_cls: The Agent class that registered the Intent
+        intent_cls: The Intent to load language data for
+        language_code: A specific language code to load. If not present, all
+            available languages will be returned
+    """
+    if "__intent_language_data__" in intent_cls.__dict__:
+        return intent_cls.__intent_language_data__
+        
     try:
         language_folder = agent_language.agent_language_folder(agent_cls)
 
@@ -432,14 +480,17 @@ def _build_responses(responses_data: dict):
         try:
             response_group = IntentResponseGroup(response_group)
         except ValueError as exc:
-            raise NotImplementedError(f"Unsupported Response Group '{response_group}' in 'responses'. Currently, only 'default' and 'rich' are supported") from exc
+            raise NotImplementedError(f"Unsupported Response Group '{response_group}' in " +
+                "'responses'. Currently, only 'default' and 'rich' are supported") from exc
 
         result[response_group] = []
         for r in responses:
             assert len(r) == 1
             for r_type, r_data in r.items():
                 if response_group == IntentResponseGroup.DEFAULT and r_type != 'text':
-                    raise ValueError(f"Message type {r_type} found in response group 'default'. Only 'text' type is allowed in 'default': please define the additional 'rich' response group to use rich responses.")
+                    raise ValueError(f"Message type {r_type} found in response group 'default'. " +
+                        "Only 'text' type is allowed in 'default': please define the additional " +
+                        "'rich' response group to use rich responses.")
 
                 if r_type == 'text':
                     result[response_group].append(TextIntentResponse.from_yaml(r_data))
@@ -452,6 +503,7 @@ def _build_responses(responses_data: dict):
                 elif r_type == 'custom':
                     result[response_group].append(CustomPayloadIntentResponse.from_yaml(r_data))
                 else:
-                    raise NotImplementedError(f"Unsupported response type '{r_type}'. Currently, only 'text' is supported")
+                    raise NotImplementedError(f"Unsupported response type '{r_type}'. Currently, " +
+                        "only 'text' is supported")
                 
     return result
