@@ -1,3 +1,5 @@
+from typing import List
+from dataclasses import dataclass
 from unittest.mock import patch, call
 
 import pytest
@@ -20,18 +22,18 @@ def test_register_intent_valid_language_data(mock_language):
     mock_language.LanguageCode = real_LanguageCode
 
     MyAgent = _get_toy_agent()
-    MyAgent._register_intent(smalltalk.hello)
+    MyAgent._register_intent(smalltalk.Hello)
 
-    assert MyAgent.intents == [smalltalk.hello]
-    assert MyAgent._intents_by_name == {'smalltalk.hello': smalltalk.hello}
-    assert MyAgent._intents_by_event == {'E_SMALLTALK_HELLO': smalltalk.hello}
+    assert MyAgent.intents == [smalltalk.Hello]
+    assert MyAgent._intents_by_name == {'smalltalk.Hello': smalltalk.Hello}
+    assert MyAgent._intents_by_event == {'E_SMALLTALK_HELLO': smalltalk.Hello}
 
     MyAgent = _get_toy_agent()
-    MyAgent.register(smalltalk.hello)
+    MyAgent.register(smalltalk.Hello)
 
-    assert MyAgent.intents == [smalltalk.hello]
-    assert MyAgent._intents_by_name == {'smalltalk.hello': smalltalk.hello}
-    assert MyAgent._intents_by_event == {'E_SMALLTALK_HELLO': smalltalk.hello}
+    assert MyAgent.intents == [smalltalk.Hello]
+    assert MyAgent._intents_by_name == {'smalltalk.Hello': smalltalk.Hello}
+    assert MyAgent._intents_by_event == {'E_SMALLTALK_HELLO': smalltalk.Hello}
 
 @patch('intents.language.intent_language_data')
 def test_register_intent_invalid_language_data(mock_language):
@@ -42,7 +44,83 @@ def test_register_intent_invalid_language_data(mock_language):
     MyAgent = _get_toy_agent()
     
     with pytest.raises(ValueError):
-        MyAgent._register_intent(smalltalk.hello)
+        MyAgent._register_intent(smalltalk.Hello)
+
+@patch('intents.language.intent_language_data')
+def test_register_intent_non_unique_name(mock_language):
+    @dataclass
+    class AnIntent(Intent):
+        pass
+
+    @dataclass
+    class an_intent(Intent):
+        pass
+
+    @dataclass
+    class Anintent(Intent):
+        pass
+
+    @dataclass
+    class AnIntentWithCustomName(Intent):
+        name = f"agent_test.An.Intent"
+
+    class MyAgent(Agent):
+        pass
+
+    MyAgent.register(AnIntent)
+
+    with pytest.raises(ValueError):
+        MyAgent.register(an_intent)
+
+    with pytest.raises(ValueError):
+        MyAgent.register(Anintent)
+
+    with pytest.raises(ValueError):
+        MyAgent.register(AnIntentWithCustomName)
+
+@patch('intents.language.intent_language_data')
+def test_register_intent_non_unique_parameter(mock_language):
+    @dataclass
+    class an_intent(Intent):
+        foo: Sys.Person
+
+    @dataclass
+    class a_consistent_intent(Intent):
+        foo: Sys.Person
+        bar: Sys.Integer
+
+    @dataclass
+    class an_inconsistent_intent(Intent):
+        foo: Sys.Integer
+        bar: Sys.Integer
+
+    @dataclass
+    class another_inconsistent_intent(Intent):
+        foo: List[Sys.Person]
+
+    class MyAgent(Agent):
+        pass
+
+    MyAgent.register(an_intent)
+    MyAgent.register(a_consistent_intent)
+    
+    with pytest.raises(ValueError):
+        MyAgent.register(an_inconsistent_intent)
+
+    with pytest.raises(ValueError):
+        MyAgent.register(another_inconsistent_intent)
+
+@patch('intents.language.intent_language_data')
+def test_register_intent_invalid_param_schema_list_default(mock_language):
+    MyAgent = _get_toy_agent()
+    
+    with pytest.raises(ValueError):
+        @dataclass
+        class intent_with_invalid_list_default(Intent):
+            """Intent with invalid parameter default"""
+            optional_list_param: List[Sys.Person] = 42
+        
+        MyAgent.register(intent_with_invalid_list_default)
 
 @patch('intents.model.agent.language')
 def test_register_intent_registers_entities(mock_language):
@@ -53,6 +131,7 @@ def test_register_intent_registers_entities(mock_language):
     class MyEntity(Entity):
         pass
 
+    @dataclass
     class MyIntent(Intent):
         name = "my_intent"
 
@@ -66,6 +145,7 @@ def test_register_intent_registers_entities(mock_language):
     assert MyAgent._intents_by_event == {'E_MY_INTENT': MyIntent}
     assert MyAgent._entities_by_name == {'MyEntity': MyEntity}
 
+    @dataclass
     class MyOtherIntent(Intent):
         name = "my_other_intent"
 
@@ -84,6 +164,7 @@ def test_register_intent_registers_entities(mock_language):
             pass
         return MyEntity
 
+    @dataclass
     class MyInvalidIntent(Intent):
         name = "my_invalid_intent"
 
@@ -100,12 +181,9 @@ def test_register_module(mock_language):
     with patch.object(MyAgent, '_register_intent') as mock_register_intent:
         MyAgent.register(smalltalk)
         mock_register_intent.asssert_has_calls([
-            call(MyAgent, smalltalk.hello),
-            call(MyAgent, smalltalk.user_name_give),
-            call(MyAgent, smalltalk.agent_name_give),
-            call(MyAgent, smalltalk.user_likes_music),
-            call(MyAgent, smalltalk.greet_friends),
-            call(MyAgent, smalltalk.agent_welcomes_user),
+            call(MyAgent, smalltalk.Hello),
+            call(MyAgent, smalltalk.UserNameGive),
+            call(MyAgent, smalltalk.AgentNameGive),
+            call(MyAgent, smalltalk.UserLikesMusic),
+            call(MyAgent, smalltalk.GreetFriends),
         ])
-
-# test duplicate intent
