@@ -23,7 +23,7 @@ from dataclasses import dataclass
 class _EntityMetaclass(type):
 
     name: str = None
-    custom_language_data: Dict["language.LanguageCode", List["language.EntityEntry"]] = None
+    __entity_language_data__: Dict["language.LanguageCode", List["language.EntityEntry"]] = None
 
     def __new__(cls, name, bases, dct):
         result_cls = super().__new__(cls, name, bases, dct)
@@ -80,10 +80,10 @@ class Entity(str, EntityMixin):
         from intents import Intent, Entity
 
         class PizzaType(Entity):
-            \"\"\"One of the pizza types that a Customer can order\"\"\" 
+            \"\"\"One of the pizza types that a Customer can order\"\"\"
 
         @dataclass
-        class customer_orders_pizza(Intent):
+        class CustomerOrdersPizza(Intent):
             \"\"\"A little docstring for my Intent\"\"\"
 
             pizza_type: PizzaType
@@ -102,8 +102,16 @@ class Entity(str, EntityMixin):
         """
         This class is used to set user-controlled Entity parameters.
 
-        **NOTE**: usage of Meta is experimental, it may be removed in upcoming
-        releases. 
+        .. warning::
+
+            The use of Meta is strictly bound to Dialogflow; support
+            may be dropped in the next releases of Intents
+
+        Args:
+            regex_entity: Whether to treat entity values as regular expressions
+            automated_expansion: Service will try to match synonyms even if they
+                are not explicitly declared
+            fuzzy_matching: Service will ignore word order in matching
         """
         regex_entity: bool = False
         automated_expansion: bool = False
@@ -129,11 +137,21 @@ class Sys:
         from intents import Intent, Sys
 
         @dataclass
-        class user_says_name(Intent):
+        class UserSaysName(Intent):
             \"\"\"A little docstring for my Intent\"\"\"
 
             user_name: Sys.Person
 
+    System entities inherit from Python base types, so
+
+    .. code-block:: python
+
+        >>> isinstance(Sys.Integer(42), int)
+        True
+        >>> from datetime import date
+        >>> isinstance(Sys.Date.from_py_date(date(2021, 7, 30)), date)
+        True
+
     **NOTE**: This class can be expanded to contain more sophisticated entities,
     such as time intervals, amounts with measurement units and such. These may
     not be natively supported on every prediction service: some attention is
@@ -158,6 +176,9 @@ class Sys:
             """
             Clone the given :class:`datetime.date` object into a `Sys.Date`
             object. This is mostly for internal use.
+
+            Args:
+                py_date: A Python date object
             """
             return Sys.Date(py_date.year, py_date.month, py_date.day)
 
@@ -171,6 +192,9 @@ class Sys:
             """
             Clone the given :class:`datetime.time` object into a `Sys.Time`
             object. This is mostly for internal use.
+
+            Args:
+                py_time: A Python time object
             """
             return Sys.Time(py_time.hour, py_time.minute, py_time.second, tzinfo=py_time.tzinfo)
 
