@@ -1,7 +1,7 @@
 """
 An **intent** is a categorical representation of the User intention in a single
 conversation turn. For instance, utterances like "I want a pizza", "I'd like to
-order a pizza" and such, could be mapped to a single `order_pizza` intent.
+order a pizza" and such, could be mapped to a single `OrderPizza` intent.
 
 Your agent will typically define a number of *intents*, representing all the
 types of messages the Agent can understand and answer to. This is done by
@@ -11,13 +11,14 @@ class with :meth:`intents.Agent.register`.
 """
 
 import re
+import inspect
 import logging
 import dataclasses
-from dataclasses import dataclass, is_dataclass
+from dataclasses import dataclass
 from typing import List, Dict, Any, _GenericAlias
 
 from intents.model import context, event, entity
-from intents import language
+from intents.helpers.data_classes import is_dataclass_strict
 
 logger = logging.getLogger(__name__)
 
@@ -27,12 +28,38 @@ logger = logging.getLogger(__name__)
 
 @dataclass
 class IntentParameterMetadata:
+    """
+    Model metadata of a single Intent parameter. `IntentParameterMetadata`
+    objects are built internally by :meth:`Intent.parameter_schema` based on the
+    Intent dataclass fields.
+
+    Args:
+        name: Parameter name
+        entity_cls: Parameter type
+        is_list: Parameter will match multiple values in the User utterance
+        required: If True, user will be prompted for parameter value when that
+            can't be tagged in his utterance
+        default: If set, this value will be used when parameter value can't be
+            tagged in the User's utterance
+    """
     name: str
     entity_cls: entity._EntityMetaclass
     is_list: bool
     required: bool
     default: Any
 
+class _CallableDict(dict):
+    """
+    This is a proxy to handle the deprecation of
+    :meth:`Intent.parameter_schema`, which is now a property. Legacy code may
+    still call it as a method (e.g. `hello_intent.parameter_schema()`), hence we
+    need to support `__call__`, warning User to update his code (e.g. `hello_intent.parameter_schema`) 
+    """
+
+    def __call__(self):
+        logger.warning("'Intent.parameter_schema()' is deprecated, and is now a property. Use 'Intent.parameter_schema` instead. Support will be removed in 0.3")
+        return self
+
 class _IntentMetaclass(type):
 
     name: str = None
@@ -48,12 +75,15 @@ class _IntentMetaclass(type):
             assert not bases
             return result_cls
 
-        if not result_cls.name:
+        for base_cls in bases:
+            if base_cls is not Intent:
+                if not is_dataclass_strict(base_cls):
+                    logger.warning("Intent '%s' is inheriting from non-dataclass '%s'. Did you forget the decorator?", result_cls, base_cls)
+
+        if "name" not in result_cls.__dict__:
             result_cls.name = _intent_name_from_class(result_cls)
-        else:
-            is_valid, reason = _is_valid_intent_name(result_cls.name)
-            if not is_valid:
-                raise ValueError(f"Invalid intent name '{result_cls.name}': {reason}")
+
+        check_intent_name(result_cls.name)
 
         if not result_cls.input_contexts:
             result_cls.input_contexts = []
@@ -69,127 +99,9 @@ class _IntentMetaclass(type):
             events.append(event_cls)
         result_cls.events = events
 
-        if not is_dataclass(result_cls):
-            result_cls = dataclass(result_cls)
-
-        # Check parameters
-        result_cls.parameter_schema()
-
         return result_cls
 
-class Intent(metaclass=_IntentMetaclass):
-    """
-    Represents a predicted intent. This is also used as a base class for the
-    intent classes that model a Dialogflow Agent in Python code.
-
-    In its simplest form, an Intent can be defined as follows:
-
-    .. code-block:: python
-
-        from intents import Intent
-
-        class user_says_hello(Intent):
-            \"\"\"A little docstring for my Intent\"\"\"
-
-    *Intents* will then look for language resources in the folder where your
-    Agent class is defined, and specifically in
-    `language/<LANGUAGE-CODE>/user_says_hello.yaml`. More details in
-    :mod:`intents.language`.
-
-    Intents can be more complex than this, for instance:
-
-    .. code-block:: python
-
-        from dataclasses import dataclass
-        from intents import Intent, Sys
-
-        @dataclass
-        class user_says_hello(Intent):
-            \"\"\"A little docstring for my Intent\"\"\"
-
-            user_name: Sys.Person
-
-            name = "hello_custom_name"
-            input_contexts = [a_context]
-            input_contexts = [a_context(2), another_context(1)]
-
-    This Intent has a custom name (i.e. will appear as "hello_custom_name" when
-    exported to Dialogflow), will be predicted only when `a_context` is active,
-    and will spawn `a_context`, lasting 2 conversation turns, and
-    `another_context` lasting only 1 conversation turn.
-
-    Most importantly, this intent has a `user_name` **parameter** of type
-    :class:`Sys.Person` (check out :class:`intents.model.entity.Sys` for available system
-    entities). With adequate examples in its language file, it will be able to
-    match utterances like "Hello, my name is John", tagging "John" as an Entity.
-    When a connector is instantiated, predictions will look like this:
-
-    >>> predicted = connector.predict("My name is John")
-    user_says_hello(user_name="John") predicted.user_name "John"
-    predicted.fulfillment_text "Hi John, I'm Agent"
-
-    Last, we notice the **@dataclass** decorator. This isn't really needed for
-    the Intent to work, but adding it will have your IDE recognize the Intent
-    class as a dataclass: you want autocomplete and static type checking when
-    working with hundreds of intents in the same project.
-    """
-    # TODO: check parameter names: no underscore, no reserved names, max length
-
-    name: str = None
-    input_contexts: List[context._ContextMetaclass] = None
-    output_contexts: List[context.Context] = None
-    events: List[event.Event] = None # TODO: at some point this may contain strings
-
-    # A :class:`Connector` provides this
-    prediction: 'intents.Prediction'
-
     @property
-    def confidence(self) -> float:
-        return self.prediction.confidence
-
-    @property
-    def contexts(self) -> list:
-        return self.prediction.contexts
-
-    @property
-    def fulfillment_text(self) -> str:
-        return self.prediction.fulfillment_text
-
-    def fulfillment_messages(
-        self,
-        response_group: "language.IntentResponseGroup"=language.IntentResponseGroup.RICH
-    ) -> List["language.IntentResponse"]:
-        """
-        Return a list of fulfillment messages that are suitable for the given
-        Response Group. The following scenarios may happen:
-
-        * :class:`language.IntentResponseGroup.DEFAULT` is requested -> Message
-          in the `DEFAULT` group will be returned
-        * :class:`language.IntentResponseGroup.RICH` is requested
-
-            * `RICH` messages are defined -> `RICH` messages are returned
-            * No `RICH` message is defined -> `DEFAULT` messages are returned
-
-        If present, messages in the "rich" group will be returned:
-
-        >>> result.fulfillment_messages()
-        [TextIntentResponse(choices=['I like travelling too! How can I help?']),
-         QuickRepliesIntentResponse(replies=['Recommend a hotel', 'Send holiday photo', 'Where the station?'])]
-         
-        Alternatively, I can ask for plain-text default messages:
-
-        >>> from intents.language import IntentResponseGroup
-        >>> result.fulfillment_messages(IntentResponseGroup.DEFAULT)
-        [TextIntentResponse(choices=['Nice, I can send you holiday pictures, or recommend an hotel'])]
-        
-        """
-        if response_group == language.IntentResponseGroup.RICH and \
-           not self.prediction.fulfillment_messages.get(response_group):
-            response_group = language.IntentResponseGroup.DEFAULT
-
-        return self.prediction.fulfillment_messages.get(response_group, [])
-
-    @classmethod
     def parameter_schema(cls) -> Dict[str, IntentParameterMetadata]:
         """
         Return a dict representing the Intent parameter definition. A key is a
@@ -197,9 +109,20 @@ class Intent(metaclass=_IntentMetaclass):
 
         TODO: consider computing this in metaclass to cache value and check types
         """
+        if cls is Intent:
+            return {}
+
+        if not is_dataclass_strict(cls):
+            logger.warning(f"{cls} is not a dataclass. This may cause unexpected behavior: consider adding a @dataclass decorator to your Intent class.")
+            cls = dataclass(cls)
+
         result = {}
-        for param_field in cls.__dict__['__dataclass_fields__'].values():
+        for param_field in cls.__dataclass_fields__.values():
+        # for param_field in cls.__dict__['__dataclass_fields__'].values():
             # List[...]
+            if inspect.isclass(param_field.type) and issubclass(param_field.type, Intent):
+                continue
+
             if isinstance(param_field.type, _GenericAlias):
                 if param_field.type.__dict__.get('_name') != 'List':
                     raise ValueError(f"Invalid typing '{param_field.type}' for parameter '{param_field.name}'. Only 'List' is supported.")
@@ -214,6 +137,9 @@ class Intent(metaclass=_IntentMetaclass):
                 entity_cls = param_field.type
                 is_list = False
 
+            if not issubclass(entity_cls, entity.EntityMixin):
+                raise ValueError(f"Parameter '{param_field.name}' of intent '{cls.name}' is of type '{entity_cls}', which is not an Entity.")
+
             required = True
             default = None
             if not isinstance(param_field.default, dataclasses._MISSING_TYPE):
@@ -234,46 +160,130 @@ class Intent(metaclass=_IntentMetaclass):
                 default=default
             )
 
-        return result
+        return _CallableDict(result)
 
-    @classmethod
-    def from_prediction(cls, prediction: 'intents.Prediction') -> 'Intent':
-        """
-        Build an :class:`Intent` class from a :class:`Prediction`. In practice:
+class Intent(metaclass=_IntentMetaclass):
+    """
+    Represents a predicted intent. This is also used as a base class for the
+    intent classes that model a Dialogflow Agent in Python code.
 
-        #. Match parameters givent the Intent schema
-        #. Instantiate the Intent
-        #. Set the `prediction` field on the instantiated Intent.
+    In its simplest form, an Intent can be defined as follows:
 
-        Note that this method is mostly for internal use, *connectors* will call
-        it for you.
-        """
-        try:
-            parameters = prediction.parameters(cls.parameter_schema())
-        except ValueError as exc:
-            raise ValueError(f"Failed to match parameters for Intent class '{cls}'. Prediction: {prediction}") from exc
+    .. code-block:: python
 
-        result = cls(**parameters)
-        result.prediction = prediction
-        return result
+        from intents import Intent
 
-def _is_valid_intent_name(candidate_name):
+        class UserSaysHello(Intent):
+            \"\"\"A little docstring for my Intent\"\"\"
+
+    *Intents* will then look for language resources in the folder where your
+    Agent class is defined, and specifically in
+    `language/<LANGUAGE-CODE>/smalltalk.user_says_hello.yaml`. Note that the
+    name of the module where the Intent is defined (`smalltalk.py`) is used as a
+    prefix. More details in :mod:`intents.language`.
+
+    Intents can be more complex than this, for instance:
+
+    .. code-block:: python
+
+        from dataclasses import dataclass
+        from intents import Intent, Sys
+
+        @dataclass
+        class UserSaysHello(Intent):
+            \"\"\"A little docstring for my Intent\"\"\"
+
+            user_name: Sys.Person
+
+            name = "hello_custom_name"
+
+    This Intent has a custom name, so it will appear as "hello_custom_name" when
+    exported to Dialogflow, and its language file will just be
+    `hello_custom_name.yaml`, without module prefix. See
+    :func:`check_intent_name` for more details on the rules that intent names
+    must follow.
+
+    Most importantly, this intent has a `user_name` **parameter** of type
+    :class:`Sys.Person` (check out :class:`intents.model.entity.Sys` for
+    available system entities). With adequate examples in its language file, it
+    will be able to match utterances like "Hello, my name is John", tagging
+    "John" as an Entity. When a connector is instantiated, predictions will look
+    like this:
+
+    >>> predicted = connector.predict("My name is John")
+    >>> predicted.intent
+    user_says_hello(user_name="John")
+    >>> predicted.intent.user_name
+    "John"
+    >>> predicted.fulfillment_text
+    "Hi John, I'm Agent"
+
+    .. warning::
+
+        Parameter names are meant to be **unique** within an Agent. That is, the same parameter
+        must always be defined with the same type. This is because 1) Some services (e.g. 
+        `Alexa <https://developer.amazon.com/en-US/docs/alexa/custom-skills/create-the-interaction-model-for-your-skill.html#intent-slot-names>`_)
+        require so. 2) Predictions overwrite parameters in context based on their name: better
+        make sure they are compatible.
+
+    Last, we notice the **@dataclass** decorator. We require it to be added
+    explicitly, this way IDEs will recognize the Intent class as a dataclass and
+    provide correct hints for names and types.
+    """
+    # TODO: check parameter names: no underscore, no reserved names, max length
+
+    name: str = None
+    input_contexts: List[context._ContextMetaclass] = None
+    output_contexts: List[context.Context] = None
+    events: List[event.Event] = None # TODO: at some point this may contain strings
+
+    @property
+    def parameter_schema(self) -> Dict[str, IntentParameterMetadata]:
+        return self.__class__.parameter_schema
+
+def check_intent_name(candidate_name: str):
+    """
+    Raise `ValueError` if the given Intent name is not a valid name. Valid names
+
+    * Only contain letter, underscore (`_`) and period (`.`) characters
+    * Start with a letter
+    * Don't contain repeated underscores (e.g. `__`)
+    * Don't start wit `i_`. This is a reserved prefix for *Intents* system
+      intents
+    
+    Note that `Agent.register` will apply further checks to spot duplicate
+    intent names. Note that names are case insensitive, and shouldn't overlap
+    with parameter names.
+
+    Args:
+        candidate_name: The Intent name to check
+    """
+    invalid_reason = None
+    
     if re.search(r'[^a-zA-Z_\.]', candidate_name):
-        return False, "must only contain letters, underscore or dot"
+        invalid_reason = "must only contain letters, underscore or period"
 
     if candidate_name.startswith('.') or candidate_name.startswith('_'):
-        return False, "must start with a letter"
+        invalid_reason = "must start with a letter"
 
     if "__" in candidate_name:
-        return False, "must not contain __"
+        invalid_reason = "must not contain __"
 
-    return True, None
+    if candidate_name.startswith("i_"):
+        invalid_reason = "the 'i_' prefix is reserved for system intents"
+
+    if invalid_reason:
+        raise ValueError(f"Invalid intent name '{candidate_name}': {invalid_reason}. " +
+            "If the issue is related to your class name or path you can either change names to " +
+            "be compliant, or use a custom name by setting 'Intent.name' manually. See the " +
+            "documentation at https://intents.readthedocs.io/ for more information on intent " +
+            "naming rules.")
 
 def _intent_name_from_class(intent_cls: _IntentMetaclass) -> str:
     full_name = f"{intent_cls.__module__}.{intent_cls.__name__}"
-    if "__" in full_name:
-        logger.warning("Intent class '%s' contains repeated '_'. This is reserved: repeated underscores will be reduced to one, this may cause unexpected behavior.")
-    full_name = re.sub(r"_+", "_", full_name)
+    if intent_cls.__module__.startswith("_"):
+        full_name = intent_cls.__name__
+    # full_name = re.sub(r"_+", "_", full_name)
     return ".".join(full_name.split(".")[-2:])
 
 def _system_event(intent_name: str) -> str:
